==> techne-0.2.2.tar.gz: techne-0.2.2/src/polar.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "polar.h"

@implementation Polar

-(Joint *) init
{
    char *list[] = {
	"anchor", "axes", "motor", "state", "stops", "tolerance"
    };

    self->joint = dJointCreatePU (_WORLD, NULL);

    self->anchor[0] = 0;
    self->anchor[1] = 0;
    self->anchor[2] = 0;

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;

    self->axes[2][0] = 0;
    self->axes[2][1] = 0;
    self->axes[2][2] = 1;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    self->stops[2][0] = -dInfinity;
    self->stops[2][1] = dInfinity;

    [super init];
    [self add: sizeof (list) / sizeof (char *) Properties: list];

    return self;
}

-(void) update
{
    [super update];

    dJointSetPUAnchor ([self joint],
		       self->anchor[0],
		       self->anchor[1],
		       self->anchor[2]);

    dJointSetPUAxis1 ([self joint],
		      self->axes[0][0],
		      self->axes[0][1],
		      self->axes[0][2]);

    dJointSetPUAxis2 ([self joint],
		      self->axes[1][0],
		      self->axes[1][1],
		      self->axes[1][2]);

    dJointSetPUAxis3 ([self joint],
		      self->axes[2][0],
		      self->axes[2][1],
		      self->axes[2][2]);
}

-(void) get
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -1);

    if (!xstrcmp(k, "anchor")) {
	dJointGetPUAnchor ([self joint], self->anchor);

        lua_newtable (_L);
        
        for(j = 0; j < 3; j += 1) {
            lua_pushnumber (_L, self->anchor[j]);
            lua_rawseti (_L, -2, j + 1);
        }
    } else if (!xstrcmp(k, "axes")) {
	dJointGetPUAxis1 ([self joint], self->axes[0]);
	dJointGetPUAxis2 ([self joint], self->axes[1]);
	dJointGetPUAxis3 ([self joint], self->axes[2]);

	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber (_L, self->axes[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "motor")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->motor[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "stops")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);

	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->stops[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 1);
	
	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->hardness[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 2);

	    lua_pushnumber (_L, bounce[j]);
	    lua_rawseti (_L, -2, 3);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "tolerance")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_pushnumber (_L, self->tolerance[j]);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "state")) {
	dReal state[6];

	if ([self joint]) {
	    state[0] = dJointGetPUAngle1 ([self joint]);
	    state[1] = dJointGetPUAngle2 ([self joint]);
	    state[2] = dJointGetPUPosition ([self joint]);
	    state[3] = dJointGetPUAngle1Rate ([self joint]);
	    state[4] = dJointGetPUAngle2Rate ([self joint]);
	    state[5] = dJointGetPUPositionRate ([self joint]);

	    lua_newtable (_L);
        
	    for(j = 0 ; j < 6 ; j += 1) {
		lua_pushnumber (_L, state[j]);
		
		lua_rawseti (_L, -2, j + 1);
	    }
	} else {
	    lua_pushnil (_L);
	}
    } else {
	[super get];
    }
}

-(void) set
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -2);

    if (!xstrcmp(k, "anchor")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->anchor[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);
            }

	    dJointSetPUAnchor ([self joint],
			       self->anchor[0],
			       self->anchor[1],
			       self->anchor[2]);
        }
    } else if (!xstrcmp(k, "axes")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->axes[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }
		}

		dSafeNormalize3 (self->axes[j]);

		lua_pop (_L, 1);
	    }

	    dJointSetPUAxis1 ([self joint],
			      self->axes[0][0],
			      self->axes[0][1],
			      self->axes[0][2]);

	    dJointSetPUAxis2 ([self joint],
			      self->axes[1][0],
			      self->axes[1][1],
			      self->axes[1][2]);

	    dJointSetPUAxis3 ([self joint],
			      self->axes[2][0],
			      self->axes[2][1],
			      self->axes[2][2]);
	}
    } else if (!xstrcmp(k, "motor")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->motor[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dJointSetPUParam ([self joint],
				      dParamVel + dParamGroup * j,
				      self->motor[j][0]);
		    
		    dJointSetPUParam ([self joint],
				      dParamFMax + dParamGroup * j,
				      self->motor[j][1]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "stops")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    lua_rawgeti (_L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber (_L, -1);
		
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);

		    lua_rawgeti (_L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber (_L, -1);
			
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);
	
		    lua_rawgeti (_L, -1, 3);
		    self->bounce[j] = lua_tonumber (_L, -1);
		    lua_pop (_L, 1);


		    printf ("%d: %f, %f, %f, %f, %f\n", j,
			    self->stops[j][0],
			    self->stops[j][1],
			    self->hardness[j][0],
			    self->hardness[j][1],
			    self->bounce[j]);
		    /* Resetting the self->stops makes sure that lo remains
		       smaller than hi between calls. */

		    dJointSetPUParam ([self joint],
				      dParamLoStop + dParamGroup * j,
				      -dInfinity);
		    
		    dJointSetPUParam ([self joint],
				      dParamHiStop + dParamGroup * j,
				      dInfinity);
		    
		    dJointSetPUParam ([self joint],
				      dParamLoStop + dParamGroup * j,
				      self->stops[j][0]);

		    dJointSetPUParam ([self joint],
				      dParamHiStop + dParamGroup * j,
				      self->stops[j][1]);

		    dJointSetPUParam ([self joint],
				      dParamStopCFM + dParamGroup * j,
				      self->hardness[j][0]);

		    dJointSetPUParam ([self joint],
				      dParamStopERP + dParamGroup * j,
				      self->hardness[j][1]);

		    dJointSetPUParam ([self joint],
				      dParamBounce + dParamGroup * j,
				      self->bounce[j]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "tolerance")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->tolerance[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);

		dJointSetPUParam ([self joint],
				  dParamCFM + dParamGroup * j,
				  self->tolerance[j]);
            }
        }
    } else {
	[super set];
    }
}

@end

==> techne-0.2.3.tar.gz: techne-0.2.3/src/polar.c <==
/* Copyright (C) 2009 Papavasileiou Dimitris                             
 *                                                                      
 * This program is free software: you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License as published by 
 * the Free Software Foundation, either version 3 of the License, or    
 * (at your option) any later version.                                  
 *                                                                      
 * This program is distributed in the hope that it will be useful,      
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        
 * GNU General Public License for more details.                         
 *                                                                      
 * You should have received a copy of the GNU General Public License    
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <ode/ode.h>
#include <lua.h>
#include <lualib.h>
#include <lauxlib.h>

#include "polar.h"

@implementation Polar

-(Joint *) init
{
    char *list[] = {
	"anchor", "axes", "motor", "state", "stops", "tolerance"
    };

    self->joint = dJointCreatePU (_WORLD, NULL);

    self->anchor[0] = 0;
    self->anchor[1] = 0;
    self->anchor[2] = 0;

    self->axes[0][0] = 1;
    self->axes[0][1] = 0;
    self->axes[0][2] = 0;

    self->axes[1][0] = 0;
    self->axes[1][1] = 1;
    self->axes[1][2] = 0;

    self->axes[2][0] = 0;
    self->axes[2][1] = 0;
    self->axes[2][2] = 1;
    
    self->stops[0][0] = -dInfinity;
    self->stops[0][1] = dInfinity;

    self->stops[1][0] = -dInfinity;
    self->stops[1][1] = dInfinity;

    self->stops[2][0] = -dInfinity;
    self->stops[2][1] = dInfinity;

    [super init];
    [self add: sizeof (list) / sizeof (char *) Properties: list];

    return self;
}

-(void) update
{
    [super update];

    dJointSetPUAnchor ([self joint],
		       self->anchor[0],
		       self->anchor[1],
		       self->anchor[2]);

    dJointSetPUAxis1 ([self joint],
		      self->axes[0][0],
		      self->axes[0][1],
		      self->axes[0][2]);

    dJointSetPUAxis2 ([self joint],
		      self->axes[1][0],
		      self->axes[1][1],
		      self->axes[1][2]);

    dJointSetPUAxis3 ([self joint],
		      self->axes[2][0],
		      self->axes[2][1],
		      self->axes[2][2]);
}

-(void) get
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -1);

    if (!xstrcmp(k, "anchor")) {
	dJointGetPUAnchor ([self joint], self->anchor);

        lua_newtable (_L);
        
        for(j = 0; j < 3; j += 1) {
            lua_pushnumber (_L, self->anchor[j]);
            lua_rawseti (_L, -2, j + 1);
        }
    } else if (!xstrcmp(k, "axes")) {
	dJointGetPUAxis1 ([self joint], self->axes[0]);
	dJointGetPUAxis2 ([self joint], self->axes[1]);
	dJointGetPUAxis3 ([self joint], self->axes[2]);

	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 3; i += 1) {
		lua_pushnumber (_L, self->axes[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "motor")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);
        
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->motor[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "stops")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_newtable (_L);

	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->stops[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 1);
	
	    lua_newtable (_L);
	    for(i = 0; i < 2; i += 1) {
		lua_pushnumber (_L, self->hardness[j][i]);
		lua_rawseti (_L, -2, i + 1);
	    }
	    lua_rawseti (_L, -2, 2);

	    lua_pushnumber (_L, bounce[j]);
	    lua_rawseti (_L, -2, 3);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "tolerance")) {
	lua_newtable (_L);
        
	for(j = 0 ; j < 3 ; j += 1) {
	    lua_pushnumber (_L, self->tolerance[j]);

	    lua_rawseti (_L, -2, j + 1);
	}
    } else if (!xstrcmp(k, "state")) {
	dReal state[6];

	if ([self joint]) {
	    state[0] = dJointGetPUAngle1 ([self joint]);
	    state[1] = dJointGetPUAngle2 ([self joint]);
	    state[2] = dJointGetPUPosition ([self joint]);
	    state[3] = dJointGetPUAngle1Rate ([self joint]);
	    state[4] = dJointGetPUAngle2Rate ([self joint]);
	    state[5] = dJointGetPUPositionRate ([self joint]);

	    lua_newtable (_L);
        
	    for(j = 0 ; j < 6 ; j += 1) {
		lua_pushnumber (_L, state[j]);
		
		lua_rawseti (_L, -2, j + 1);
	    }
	} else {
	    lua_pushnil (_L);
	}
    } else {
	[super get];
    }
}

-(void) set
{
    const char *k;
    int i, j;
    
    k = lua_tostring (_L, -2);

    if (!xstrcmp(k, "anchor")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->anchor[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);
            }

	    dJointSetPUAnchor ([self joint],
			       self->anchor[0],
			       self->anchor[1],
			       self->anchor[2]);
        }
    } else if (!xstrcmp(k, "axes")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 3 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->axes[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }
		}

		dSafeNormalize3 (self->axes[j]);

		lua_pop (_L, 1);
	    }

	    dJointSetPUAxis1 ([self joint],
			      self->axes[0][0],
			      self->axes[0][1],
			      self->axes[0][2]);

	    dJointSetPUAxis2 ([self joint],
			      self->axes[1][0],
			      self->axes[1][1],
			      self->axes[1][2]);

	    dJointSetPUAxis3 ([self joint],
			      self->axes[2][0],
			      self->axes[2][1],
			      self->axes[2][2]);
	}
    } else if (!xstrcmp(k, "motor")) {
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
			self->motor[j][i] = lua_tonumber (_L, -1);
                
			lua_pop (_L, 1);
		    }

		    dJointSetPUParam ([self joint],
				      dParamVel + dParamGroup * j,
				      self->motor[j][0]);
		    
		    dJointSetPUParam ([self joint],
				      dParamFMax + dParamGroup * j,
				      self->motor[j][1]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "stops")) {
	for(j = 0 ; j < 3 ; j += 1) {
	    /* Resetting the self->stops makes sure that lo remains
	       smaller than hi between calls. */

	    dJointSetPUParam ([self joint],
			      dParamLoStop + dParamGroup * j,
			      -dInfinity);
		    
	    dJointSetPUParam ([self joint],
			      dParamHiStop + dParamGroup * j,
			      dInfinity);
	}
	
        if(lua_istable (_L, 3)) {
	    for(j = 0 ; j < 3 ; j += 1) {
		lua_rawgeti (_L, 3, j + 1);
		
		if(lua_istable (_L, -1)) {
		    lua_rawgeti (_L, -1, 1);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);
		
			self->stops[j][i] = lua_tonumber (_L, -1);
		
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);

		    lua_rawgeti (_L, -1, 2);
		    for(i = 0 ; i < 2 ; i += 1) {
			lua_rawgeti (_L, -1, i + 1);

			self->hardness[j][i] = lua_tonumber (_L, -1);
			
			lua_pop (_L, 1);
		    }
		    lua_pop (_L, 1);
	
		    lua_rawgeti (_L, -1, 3);
		    self->bounce[j] = lua_tonumber (_L, -1);
		    lua_pop (_L, 1);

		    /* printf ("%d: %f, %f, %f, %f, %f\n", j, */
		    /* 	    self->stops[j][0], */
		    /* 	    self->stops[j][1], */
		    /* 	    self->hardness[j][0], */
		    /* 	    self->hardness[j][1], */
		    /* 	    self->bounce[j]); */
		    
		    dJointSetPUParam ([self joint],
				      dParamLoStop + dParamGroup * j,
				      self->stops[j][0]);

		    dJointSetPUParam ([self joint],
				      dParamHiStop + dParamGroup * j,
				      self->stops[j][1]);

		    dJointSetPUParam ([self joint],
				      dParamStopCFM + dParamGroup * j,
				      self->hardness[j][0]);

		    dJointSetPUParam ([self joint],
				      dParamStopERP + dParamGroup * j,
				      self->hardness[j][1]);

		    dJointSetPUParam ([self joint],
				      dParamBounce + dParamGroup * j,
				      self->bounce[j]);
		}

		lua_pop (_L, 1);
	    }
	}
    } else if (!xstrcmp(k, "tolerance")) {
        if(lua_istable (_L, 3)) {
            for(j = 0 ; j < 3 ; j += 1) {
                lua_rawgeti (_L, 3, j + 1);
                self->tolerance[j] = lua_tonumber (_L, -1);
                
                lua_pop (_L, 1);

		dJointSetPUParam ([self joint],
				  dParamCFM + dParamGroup * j,
				  self->tolerance[j]);
            }
        }
    } else {
	[super set];
    }
}

@end
